﻿Imports System.Net, System.Net.Sockets, System.IO, System.Threading
Imports System.Text.Encoding
Imports System.Console

Module Module1

    Dim version As String = "0.3"

    Dim serv As Socket

    Dim verbose As Boolean = False
    Dim mustLog As Boolean = False, mustLogAll As Boolean = False

    Dim sortByID As Boolean = False, sortByIP As Boolean = False

    Dim showPrompt As Boolean = False

    Dim alreadyGot As Boolean = False

    Dim pathToCurrentDir As String = Application.StartupPath & "\Current"
    Dim pathToPKMsDir As String = Application.StartupPath & "\PKMs"

    Dim log As String = Path.ChangeExtension(Application.ExecutablePath, "log")

    Dim errorMessage As String = "This IP address is running M@T's custom Nintendo GTS server v" & version & "."

    Structure ReqStruct
        Dim sock As Socket
        Dim req As Request
    End Structure

    Sub Main(ByVal args As String())
        ForegroundColor = ConsoleColor.White
        WriteLine("*** Fake GTS server v" & version & " by M@T ***")
        WriteLine("    Based on LordLandon's sendpkm.py and Eevee's fake_gts.py")
        ResetColor()
        WriteLine()

        For Each arg As String In args
            If IsArg(arg, "verbose") OrElse IsArg(arg, "v") Then
                verbose = True

            ElseIf IsArg(arg, "log") Then
                mustLog = True

            ElseIf IsArg(arg, "logall") Then
                mustLogAll = True

            ElseIf IsArg(arg, "sortbyid") Then
                sortByID = True

            ElseIf IsArg(arg, "sortbyip") Then
                sortByIP = True

            ElseIf IsArg(arg, "prompt") Then
                showPrompt = True
            End If
        Next arg

        If showPrompt Then
            ForegroundColor = ConsoleColor.Yellow
            WriteLine(" --- Settings ---")
            ResetColor()
            WriteLine()

            If Not verbose Then
                verbose = DidUserSayYes("Do you want the program to be verbose ?")
                WriteLine()
            End If

            If (Not (mustLog Or mustLogAll)) Then
                If DidUserSayYes("Do you want to log what happens ?") Then
                    If DidUserSayYes("Do you want to log everything ?") Then
                        mustLogAll = True
                    Else
                        mustLog = True
                    End If
                End If
                WriteLine()
            End If

            If Not sortByID Then
                sortByID = DidUserSayYes("Do you want to sort the .PKM's in different folders by game ID's ?")
                WriteLine()
            End If

            If Not sortByIP Then
                sortByIP = DidUserSayYes("Do you want to sort the .PKM's by IP addresses " & If(sortByID, "too", "instead") & " ?")
                WriteLine()
            End If

            ForegroundColor = ConsoleColor.Yellow
            WriteLine(" ----------------")
            ResetColor()
            WriteLine()
        End If

        If (mustLog Or mustLogAll) AndAlso File.Exists(log) AndAlso File.ReadAllBytes(log).Length() > 0 Then
            writeToLog("----------------------------------------", False)
        End If

        If Not Directory.Exists(pathToCurrentDir) Then
            Directory.CreateDirectory(pathToCurrentDir)
            If verbose Then
                WriteLine(" * Directory 'Current' doesn't exist, creating it...")
                WriteLine()
                If mustLogAll Then writeToLog("Directory 'Current' doesn't exist, creating it...", True)
            End If
        End If

        InitializeDictionaries()

        If verbose Then
            WriteLine(" * Dictionaries initialized.")
            WriteLine()
            If mustLogAll Then writeToLog("Dictionaries initialized.", True)
        End If

        serv = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
        serv.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, 1)

        Try
            serv.Bind(New IPEndPoint(IPAddress.Any, 80))
        Catch ex As System.Net.Sockets.SocketException
            ForegroundColor = ConsoleColor.Red
            WriteLine("Error : There is already a program listening on port 80 TCP (HTTP).")
            If mustLogAll Then writeToLog("Error : There is already a program listening on port 80 TCP (HTTP). Exiting...", True)
            WriteLine("Please make sure nothing is bound to that port (usually webservers like Apache" & vbCrLf & "or IIS), then restart the program.")
            ResetColor()
            WriteLine()
            Write("Press any key to continue... ")
            ReadKey(True)
            Exit Sub
        End Try

        serv.Listen(50)

        Dim t As Thread

        WriteLine("Waiting for a connection...")
        WriteLine()
        If mustLog Or mustLogAll Then writeToLog("Server started.", True)

        While True
            t = New Thread(AddressOf asyncReq)
            t.SetApartmentState(ApartmentState.STA)
            t.Start(serv.Accept())
        End While

        WriteLine("Exiting...")
        serv.Close()
    End Sub

    Private Sub asyncReq(ByVal sk As Socket)
        Dim ansStr As String
        Dim ans As Byte() = {}
        Dim reqst As ReqStruct
        Dim data As Byte() = New Byte() {}

        Try
            reqst = GetReq(sk)
        Catch ex As Exception
            ForegroundColor = ConsoleColor.Red
            WriteLine(String.Format("Got wrong request from {0}, probably not from a DS.", CType(sk.RemoteEndPoint, IPEndPoint).Address().ToString()))
            WriteLine("Error was : " & ex.Message())
            ResetColor()
            WriteLine()

            Dim months As String() = {"???", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"}
            Dim days As String() = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"}

            Dim t As Date = Now()

            sk.Send(GetEncoding("iso-8859-1").GetBytes( _
                    "HTTP/1.1 403 Forbidden" & vbCrLf & _
                    "Date: " & days(t.DayOfWeek()) & ", " & t.Day() & " " & t.Month() & " " & t.Year() & " " & t.Hour() & ":" & t.Minute() & ":" & t.Second() & " GMT" & vbCrLf & _
                    "Server: GTS_Server" & vbCrLf & _
                    "Content-Length: " & errorMessage.Length() & vbCrLf & _
                    "Connection: Close" & vbCrLf & _
                    "Content-Type: text/html; charset=iso-8859-1" & vbCrLf & vbCrLf & _
                    errorMessage))
            sk.Close()

            Exit Sub
        End Try

        Dim sock As Socket = reqst.sock
        Dim req As Request = reqst.req
        Dim pid As String = req.getvars("pid")

        Dim clientIP As String = CType(sock.RemoteEndPoint(), IPEndPoint).Address().ToString()

        Write("Client ")
        ForegroundColor = ConsoleColor.DarkCyan
        WriteLine(clientIP)
        ResetColor()
        WriteLine("Requesting : " & req.RequestedPage())
        WriteLine()

        If mustLogAll OrElse (mustLog AndAlso (req.action = "return" Or req.action = "post")) Then
            writeToLog("Client {0} requested {1}", True, clientIP, req.RequestedPage())
        End If

        Dim currentDataFileName As String = pathToCurrentDir & "\current_" & pid & ".dat"

        If req.getvars.Count() = 1 Then
            ansStr = RandStr(32)
            If verbose Then
                WriteLine(" * Sending a random string to the client : " & ansStr)
                WriteLine()
                If mustLogAll Then writeToLog(" * Sending a random string to the client : " & ansStr, False)
            End If
        Else
            Dim encrypted_data As Byte() = System.Convert.FromBase64String(req.getvars("data").Replace("-", "+").Replace("_", "/"))
            data = Decrypt_data(encrypted_data)
            data = data.ToList().GetRange(4, data.Length() - 4).ToArray()

            Select Case req.action
                Case "info"
                    If verbose Then
                        WriteLine(" * Client is checking whether the server is responding...")
                        WriteLine()
                        If mustLogAll Then writeToLog(" * Client is checking whether the server is responding...", False)
                    End If

                    ans = New Byte() {1, 0}

                Case "setProfile"
                    If verbose Then
                        WriteLine(" * Dunno what it is used for.")
                        WriteLine()
                        If mustLogAll Then writeToLog(" * Dunno what it is used for.", False)
                    End If

                    ans = New Byte() {0, 0, 0, 0, 0, 0, 0, 0}

                Case "result"
                    If File.Exists(currentDataFileName) Then
                        'ans = File2PKMBytes("Current\current_" & pid & ".pkm")
                        If verbose Then
                            WriteLine(" * A Pokémon is present in the GTS.")
                            WriteLine()
                            If mustLogAll Then writeToLog(" * A Pokémon is present in the GTS.", False)
                        End If

                        ans = New Byte() {4, 0}
                    Else
                        If verbose Then
                            WriteLine(" * There is no Pokémon stored.")
                            WriteLine()
                            If mustLogAll Then writeToLog(" * There is no Pokémon stored.", False)
                        End If

                        ans = New Byte() {5, 0}
                    End If

                    alreadyGot = False

                Case "get"
                    If File.Exists(currentDataFileName) Then
                        ans = File.ReadAllBytes(currentDataFileName)
                    Else
                        ans = New Byte() {1, 0}
                    End If

                    If verbose Then
                        If alreadyGot Then
                            WriteLine(" * Taking the Pokémon back...")
                            If mustLogAll Then writeToLog(" * Taking the Pokémon back...", False)
                        Else
                            WriteLine(" * Checking stored Pokémon info...")
                            If mustLogAll Then writeToLog(" * Checking stored Pokémon info...", False)
                        End If
                        WriteLine()
                    End If

                    alreadyGot = True

                Case "return"
                    If File.Exists(currentDataFileName) Then
                        Dim pkmSpecie As String = dPKMSpecies(BitConverter.ToUInt16(File.ReadAllBytes(currentDataFileName), 236))

                        Write(" * Sending ")
                        ForegroundColor = ConsoleColor.Blue
                        Write(pkmSpecie)
                        ResetColor()
                        WriteLine(" back to the game...")
                        WriteLine()
                        If mustLog Or mustLogAll Then writeToLog(" * Sending {0} back to the game...", False, pkmSpecie)

                        File.Delete(currentDataFileName)

                        If verbose Then
                            WriteLine(" * Stored Pokémon file was deleted.")
                            WriteLine()
                            If mustLogAll Then writeToLog(" * Stored Pokémon file was deleted.", False)
                        End If
                    End If

                    ans = New Byte() {1, 0}

                Case "delete"
                    WriteLine(" * Sending the Pokémon to the game...")
                    WriteLine()
                    If mustLogAll Then writeToLog(" * Sending the Pokémon to the game...", False)

                    ans = New Byte() {1, 0}

                Case "post"
                    Try
                        Dim binData As Byte() = data.ToList().GetRange(0, 236).ToArray()
                        Dim postData As Byte() = data.ToList().GetRange(236, data.Length() - 236).ToArray()

                        Dim pkmData As Byte() = DecryptPokemon(binData)
                        Dim pkmSpecie As String = dPKMSpecies(BitConverter.ToUInt16(postData, 0))
                        Dim pkmLevel As Byte = postData(3)
                        Dim pkmGenderName As String

                        Select Case postData(2)
                            Case 1
                                pkmGenderName = "male"

                            Case 2
                                pkmGenderName = "female"

                            Case Else
                                pkmGenderName = "genderless"
                        End Select

                        Dim reqstedSpecie As String = dPKMSpecies(BitConverter.ToUInt16(postData, 4))
                        Dim reqstedMinLevel As Byte = postData(7)
                        Dim reqstedMaxLevel As Byte = postData(8)
                        Dim reqstedLevelStr As String

                        If reqstedMinLevel = 0 And reqstedMaxLevel = 0 Then
                            reqstedLevelStr = "any lvl"

                        ElseIf reqstedMinLevel = 0 Then
                            reqstedLevelStr = "lvl " & reqstedMaxLevel & "-"

                        ElseIf reqstedMaxLevel = 0 Then
                            reqstedLevelStr = "lvl " & reqstedMinLevel & "+"

                        ElseIf reqstedMinLevel = reqstedMaxLevel Then
                            reqstedLevelStr = "lvl " & reqstedMinLevel

                        Else
                            reqstedLevelStr = "lvl " & reqstedMinLevel & "-" & reqstedMaxLevel
                        End If

                        Dim reqstedGenderName As String

                        Select Case postData(6)
                            Case 1
                                reqstedGenderName = "male"

                            Case 2
                                reqstedGenderName = "female"

                            Case Else
                                reqstedGenderName = "either"
                        End Select

                        Dim filename As String = pathToPKMsDir

                        If sortByID Then filename &= "_" & pid
                        If sortByIP Then filename &= "_" & clientIP

                        If Not Directory.Exists(filename) Then
                            Directory.CreateDirectory(filename)

                            If verbose Then
                                WriteLine(" * Directory '" & New DirectoryInfo(filename).Name() & "' doesn't exist, creating it...")
                                WriteLine()
                                If mustLogAll Then writeToLog("Directory '" & New DirectoryInfo(filename).Name() & "' doesn't exist, creating it...", True)
                            End If
                        End If

                        filename &= "\" & Now.Ticks() & "_" & pkmSpecie & "_" & pid & ".pkm"

                        File.WriteAllBytes(filename, pkmData)
                        'File.WriteAllBytes("Current\current_" & pid & ".pkm", pkmData)
                        File.WriteAllBytes(currentDataFileName, data)

                        Write(" * A ")
                        ForegroundColor = ConsoleColor.Blue
                        Write(pkmSpecie)
                        ResetColor()
                        WriteLine(" (lvl {0}, {1}) was deposited and converted to .pkm.", pkmLevel, pkmGenderName)

                        Write("   Filename : ")
                        ForegroundColor = ConsoleColor.White
                        WriteLine(filename)
                        ResetColor()
                        WriteLine("   Requested Pokémon : {0} ({1}, {2})", reqstedSpecie, reqstedLevelStr, reqstedGenderName)
                        WriteLine()
                        If mustLog Or mustLogAll Then writeToLog(" * A {0} (lvl {1}, {2}) was deposited and converted to .pkm." & vbCrLf & _
                                                                 "   Filename : {3}" & vbCrLf & _
                                                                 "   Requested Pokémon : {4} ({5}, {6})", False, pkmSpecie, pkmLevel, pkmGenderName, filename, reqstedSpecie, reqstedLevelStr, reqstedGenderName)

                        ans = New Byte() {1, 0}
                    Catch ex As Exception
                        If verbose Then
                            ForegroundColor = ConsoleColor.Red
                            WriteLine(" * An error occured, refusing the Pokémon...")
                            ResetColor()
                            WriteLine("   Error was : " & ex.Message())
                            WriteLine()
                            If mustLogAll Then writeToLog(" * An error occured, refusing the Pokémon..." & vbCrLf & _
                                                          "   Error was : " & ex.Message(), False)
                        End If

                        ans = New Byte() {&HC, 0}
                    End Try

                Case "post_finish"
                    If verbose Then
                        WriteLine(" * Finishing the deposit operation...")
                        WriteLine()
                        If mustLogAll Then writeToLog(" * Finishing the deposit operation...", False)
                    End If

                    ans = New Byte() {1, 0}

                Case Else
                    If verbose Then
                        WriteLine(" * Unknown request, sending acknowledgement.")
                        WriteLine()
                        If mustLogAll Then writeToLog(" * Unknown request, sending acknowledgement.", False)
                    End If

                    ans = New Byte() {1, 0}
            End Select

            ansStr = GetEncoding("iso-8859-1").GetString(ans)
        End If

        SendResp(sock, ansStr)
    End Sub

    Private Function File2PKMBytes(ByVal filename As String) As Byte()
        Dim bin, pkm As Byte()

        Select Case Path.GetExtension(filename).ToLower()
            Case ".pkm"
                pkm = File.ReadAllBytes(filename)
                bin = Pokemon.EncryptPokemon(pkm.Clone())

            Case ".bin"
                bin = File.ReadAllBytes(filename)
                pkm = Pokemon.DecryptPokemon(bin.Clone())
        End Select

        ReDim Preserve pkm(235)

        Dim binEnd(55) As Byte
        Array.Copy(pkm, 8, binEnd, 0, 2) 'ID
        binEnd(2) = If((pkm(64) And 4) > 0, 3, (pkm(64) And 2) + 1) 'Gender
        binEnd(3) = pkm(140) 'Level
        Array.Copy(New Byte() {&H1, &H0, &H3, &H0, &H0, &H0, &H0, &H0}, 0, binEnd, 4, 8) 'Requesting Bulba, either, any
        Array.Copy(pkm, &H68, binEnd, 32, 16) 'OT name
        Array.Copy(pkm, &HC, binEnd, 48, 2) 'OT ID
        ReDim Preserve bin(291)
        Array.Copy(binEnd, 0, bin, 236, 56)

        Return bin
    End Function

    Private Function GetReq(ByRef sock As Socket) As ReqStruct
        'Dim sock As Socket = serv.Accept()
        Dim a(sock.ReceiveBufferSize - 1) As Byte

        Dim cnt As Integer = sock.Receive(a, sock.ReceiveBufferSize, SocketFlags.None)
        ReDim Preserve a(cnt - 1)

        Dim data As String = GetEncoding("iso-8859-1").GetString(a)

        Dim ret As ReqStruct = New ReqStruct
        ret.req = New Request(data)
        ret.sock = sock

        Return ret
    End Function

    Private Function SendResp(ByRef sock As Socket, ByVal data As Object) As Integer
        Dim resp As Response = If(TypeOf data Is Response, data, New Response(data))

        Return sock.Send(GetEncoding("iso-8859-1").GetBytes(resp.ToString))
    End Function

    Private Function RespFromServ(ByVal req As Request) As Response
        Dim s As Socket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
        s.Connect("207.38.11.146", 80)
        s.Send(GetEncoding("iso-8859-1").GetBytes(req.ToString()))

        Dim a(8191) As Byte

        Dim cnt As Integer = s.Receive(a, 8192, SocketFlags.None)
        ReDim Preserve a(cnt - 1)

        Dim data As String = GetEncoding("iso-8859-1").GetString(a)

        Return New Response(data)
    End Function

    Private Function RandStr(ByVal len As Integer) As String
        Dim pop As String = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
        RandStr = ""
        Dim rnd As Random = New Random()

        For i = 1 To len
            RandStr &= pop(rnd.Next(0, pop.Length))
        Next
    End Function

    'Private Sub serverResp()
    'Dim req As ReqStruct = GetReq()
    'Dim resp As Response = RespFromServ(req.req)
    'SendResp(req.sock, resp)
    'End Sub

    Private Function Decrypt_data(ByVal data As Byte()) As Byte()
        Dim obf_key_blob As List(Of Byte) = data.ToList().GetRange(0, 4)

        If BitConverter.IsLittleEndian Then
            obf_key_blob.Reverse()
        End If

        Dim message As Byte() = data.ToList().GetRange(4, data.Length() - 4).ToArray()
        Dim obf_key As UInt32 = BitConverter.ToUInt32(obf_key_blob.ToArray(), 0)

        Dim key As UInt32 = obf_key Xor &H4A3B2C1D

        Return Stream_decipher(message, New GTS_PRNG(key)).ToArray()
    End Function

    Private Function Stream_decipher(ByVal data As Byte(), ByVal keystream As GTS_PRNG) As List(Of Byte)
        Stream_decipher = New List(Of Byte)

        For Each c As Byte In data
            Stream_decipher.Add((c Xor keystream.NextNum()) And &HFF)
        Next
    End Function

    Private Function IsArg(ByVal givenArg As String, ByVal wantedArg As String) As Boolean
        Dim validArgs As String() = New String() {"/" & wantedArg, "-" & wantedArg, "--" & wantedArg}

        Return validArgs.Contains(givenArg.ToLower())
    End Function

    Private Function GetTime() As String
        Return "[" & Now.ToString("dd/MM/yy HH:mm:ss") & "] "
    End Function

    Private Function DidUserSayYes(Optional ByVal prompt As String = "") As Boolean
        If prompt <> "" Then
            Write(prompt & " [y/n] ")
        End If

        Dim keyChar As Char

        Do
            keyChar = ReadKey(True).KeyChar
        Loop Until New Char() {"y", "n"}.Contains(Char.ToLower(keyChar))

        WriteLine(keyChar)

        Return Char.ToLower(keyChar) = "y"
    End Function

    Private Sub writeToLog(ByVal text As String, ByVal showDateTime As Boolean)
        File.AppendAllText(log, If(showDateTime, GetTime(), "") & text & vbCrLf & vbCrLf)
    End Sub

    Private Sub writeToLog(ByVal text As String, ByVal showDateTime As Boolean, ByVal arg0 As Object)
        File.AppendAllText(log, If(showDateTime, GetTime(), "") & String.Format(text, arg0) & vbCrLf & vbCrLf)
    End Sub

    Private Sub writeToLog(ByVal text As String, ByVal showDateTime As Boolean, ByVal arg0 As Object, ByVal arg1 As Object)
        File.AppendAllText(log, If(showDateTime, GetTime(), "") & String.Format(text, arg0, arg1) & vbCrLf & vbCrLf)
    End Sub

    Private Sub writeToLog(ByVal text As String, ByVal showDateTime As Boolean, ByVal arg0 As Object, ByVal arg1 As Object, ByVal arg2 As Object)
        File.AppendAllText(log, If(showDateTime, GetTime(), "") & String.Format(text, arg0, arg1, arg2) & vbCrLf & vbCrLf)
    End Sub

    Private Sub writeToLog(ByVal text As String, ByVal showDateTime As Boolean, ByVal ParamArray args() As Object)
        File.AppendAllText(log, If(showDateTime, GetTime(), "") & String.Format(text, args) & vbCrLf & vbCrLf)
    End Sub

End Module

Class Request
    Public action, page As String
    Public getvars As Dictionary(Of String, String)

    Public Sub New()
        action = Nothing
        page = Nothing
        getvars = New Dictionary(Of String, String)
    End Sub

    Public Sub New(ByVal h As String)
        If Not (h.StartsWith("GET") AndAlso h.Contains("/pokemondpds/")) Then
            Throw New Exception("Not a DS header !")
        End If

        Dim request As String = ""

        request = h.Substring(h.IndexOf("/pokemondpds/") + 13, h.IndexOf("HTTP/1.1") - h.IndexOf("/pokemondpds/") - 14)

        page = Left(request, request.IndexOf("?"))
        action = request.Substring(request.IndexOf("/") + 1, request.IndexOf(".asp?") - request.IndexOf("/") - 1)

        Dim vars As New Dictionary(Of String, String)

        For Each i As String In request.Substring(request.IndexOf("?") + 1).Split("&")
            vars.Add(Left(i, i.IndexOf("=")), i.Substring(i.IndexOf("=") + 1))
        Next i

        getvars = vars
    End Sub

    Public Overrides Function ToString() As String
        Dim request As String = page & "?"

        For Each i As KeyValuePair(Of String, String) In getvars
            request &= i.Key & "=" & i.Value & "&"
        Next

        request = Left(request, request.Length - 1)

        Return "GET /pokemondpds/" & request & " HTTP/1.1\r\n" & _
               "Host: gamestats2.gs.nintendowifi.net\r\nUser-Agent: GameSpyHTTP/1.0\r\n" & _
               "Connection: close\r\n\r\n"
    End Function

    Public Function RequestedPage() As String
        RequestedPage = page & "?"

        For Each i As KeyValuePair(Of String, String) In getvars
            RequestedPage &= i.Key & "=" & i.Value & "&"
        Next

        RequestedPage = Left(RequestedPage, RequestedPage.Length - 1)
    End Function
End Class

Class Response
    Dim data As String
    Dim headers As List(Of String)

    Dim p3p, server, sname, cookie As String
    Dim len As Integer

    Public Sub New(ByVal h As String)
        If Not h.StartsWith("HTTP/1.1") Then
            data = h
            Return
        End If

        headers = h.Split(vbCrLf).ToList()

        Dim line As String

        Do
            line = headers(0)
            headers.RemoveAt(0)

            If line.StartsWith("P3P") Then p3p = line.Substring(line.IndexOf(": ") + 2)
            If line.StartsWith("cluster-server") Then server = line.Substring(line.IndexOf(": ") + 2)
            If line.StartsWith("X-Server-") Then sname = line.Substring(line.IndexOf(": ") + 2)
            If line.StartsWith("Content-Length") Then len = Int(line.Substring(line.IndexOf(": ") + 2))
            If line.StartsWith("Set-Cookie") Then cookie = line.Substring(line.IndexOf(": ") + 2)
        Loop Until line = ""

        data = String.Join(vbCrLf, headers.ToArray)
    End Sub

    Public Overrides Function ToString() As String
        Dim months As String() = {"???", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"}
        Dim days As String() = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"}

        Dim t As Date = Now()

        Return "HTTP/1.1 200 OK" & vbCrLf & _
               "Date: " & days(t.DayOfWeek()) & ", " & t.Day() & " " & t.Month() & " " & t.Year() & _
                    " " & t.Hour() & ":" & t.Minute() & ":" & t.Second() & " GMT" & vbCrLf & _
               "Server: Microsoft-IIS/6.0" & vbCrLf & _
               "P3P: CP='NOI ADMa OUR STP'" & vbCrLf & _
               "cluster-server: aphexweb3" & vbCrLf & _
               "X-Server-Name: AW4" & vbCrLf & _
               "X-Powered-By: ASP.NET" & vbCrLf & _
               "Content-Length: " & data.Length & vbCrLf & _
               "Content-Type: text/html" & vbCrLf & _
               "Set-Cookie: ASPSESSIONIDQCDBDDQS=JFDOAMPAGACBDMLNLFBCCNCI; path=/" & vbCrLf & _
               "Cache-control: private" & vbCrLf & vbCrLf & data
        '"Content-type: application/octet-stream" & vbCrLf & _
    End Function

    Public Function getpkm() As String()
        Dim all As List(Of String) = New List(Of String)
        Dim data_tmp As String = data
        Dim result As String = ""

        While data_tmp.Length > 0
            result = Left(data, 292)
            data = data.Substring(292)
            all.Add(Left(result, 136))
        End While

        Return all.ToArray()
    End Function
End Class